package org.example;

import com.rabbitmq.stream.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class StreamToQueueRelay {
  private static final Logger LOG = LoggerFactory.getLogger(StreamToQueueRelay.class);

  private static final String STREAM_NAME = "test_super_stream";
  private static final String CONSUMER_NAME = "stream_to_queue_relay";

  private Environment environment;
  private Consumer consumer;

  public void start() throws Exception {
    LOG.info("Starting StreamToQueueRelay");

    environment =
        Environment.builder()
            .uri("rabbitmq-stream://guest:guest@localhost:5552")
            .build();

    if (environment.streamExists(STREAM_NAME)) {
      LOG.info("Stream {} exists", STREAM_NAME);
    } else {
      LOG.error("Stream {} does not exist", STREAM_NAME);
      // Optionally, create the stream
      environment.streamCreator().stream(STREAM_NAME).create();
      LOG.info("Created stream {}", STREAM_NAME);
    }

    consumer =
        environment
            .consumerBuilder()
            .superStream(STREAM_NAME)
            .name(CONSUMER_NAME)
            .offset(OffsetSpecification.first())
            .messageHandler(this::handleMessage)
            .subscriptionListener(
                subscriptionContext -> {
                  LOG.info("Consumer started");
                })
            .build();

    LOG.info("Consumer started");
  }

  private void handleMessage(MessageHandler.Context context, Message message) {
    LOG.info("Received message: {}", new String(message.getBodyAsBinary()));
    context.processed();
  }

  public void stop() {
    LOG.info("Stopping StreamToQueueRelay");
    if (consumer != null) {
      consumer.close();
    }
    if (environment != null) {
      environment.close();
    }
  }
}
